function C = textscanu(filename, encoding)

% C = textscanu(filename, encoding) reads Unicode 
% strings from a file and outputs a cell array of strings. 
% Each row must have as many strings as the other 
% rows, the strings have to be tab-delimited, 
% and have carriage-returns at the end of rows.
% 
% Syntax:
% filename - string with the file's name and extension
%                 example: 'unicode.txt'
% encoding - encoding of the file
%                 default: UTF16-LE
% examples: UTF16-LE (little Endian), UTF8.
% See http://www.iana.org/assignments/character-sets
% MS Notepad saves in UTF16-LE, UTF16-BE or UTF8.
% 
% Note:
% Matlab's textscan function dosen't seem to handle 
% properly multiscript Unicode files. Characters 
% outside the ANSI range are given the \u001a or 
% ASCII 26 value, which usually renders on the 
% screen as a box.
% 
% Additional information at Loren on the Art of Matlab:
% http://blogs.mathworks.com/loren/2006/09/20/
% working-with-low-level-file-io-and-encodings/#comment-26764
% 
% Bug:
% When inspecting the output with the Array Editor, 
% in the Workspace or through the Command Window,
% boxes might appear instead of Unicode characters.
% Type C{1,1} at the prompt: you will see the correct
% string. Also: in Array Editor click on C then C{1,1}.
% 
% Matlab version: starting with R2006b
%
% Vlad Atanasiu * 2008.02.27
% Last revision > 2008.03.01

if nargin == 1
    encoding = 'UTF16-LE';
end
warning off MATLAB:iofun:UnsupportedEncoding;

% read input
fid = fopen(filename, 'r', 'l', encoding);
S = fscanf(fid, '%c');
fclose(fid);
% remove Byte Order Marker and add a 
% carriage-return to mark the end of the file
S = [S(2:end) char(13)]; 

% locates tab delimitators and carriage returns
tab = find(abs(S) == 9); 
crt = find(abs(S) == 13);

% get number of rows and columns in input
row = 1 + size( find(abs(S) == 10), 2);
col = 1 + size( find(abs(S) == 9), 2) / row;
C = cell(row,col); % output cell array

m = 1;
n = 1;
sos = 1; % start of string

% parse input
if col == 1
    % single column input
    for r = 1:row
        eos = crt(n) - 1; % end of string
        C(r,col) = {S(sos:eos)};
        n = n + 1;
        sos = eos + 3;
    end
else
    % multiple column input
    for r = 1:row
        for c = 1:col-1
            eos = tab(m) - 1;
            C(r,c) = {S(sos:eos)};
            sos = eos + 2;
            m = m + 1;
        end
        % last string in the row
        sos = eos + 2;
        eos = crt(n) - 1;
        C(r,col) = {S(sos:eos)};
        n = n + 1;
        sos = eos + 3;
    end
end
